@@ -44,6 +44,7 @@ double quisk_dD_out(double, struct quisk_dFilter *);
 int quisk_cInterpolate(complex double *, int, struct quisk_cFilter *, int);
 int quisk_dInterpolate(double *, int, struct quisk_dFilter *, int);
 int quisk_cDecimate(complex double *, int, struct quisk_cFilter *, int);
+int quisk_cCDecimate(complex double *, int, struct quisk_cFilter *, int);
 int quisk_dDecimate(double *, int, struct quisk_dFilter *, int);
 int quisk_cInterpDecim(complex double *, int, struct quisk_cFilter *, int, int);
 int quisk_cDecim2HB45(complex double *, int, struct quisk_cHB45Filter *);
@@ -62,7 +63,8 @@ extern double quiskFilt133D2Coefs[136];
 extern double quiskFilt167D3Coefs[174];
 extern double quiskFilt111D2Coefs[114];
 extern double quiskFilt53D1Coefs[55];
-extern double quiskFilt144D3Coefs[195];
+extern double quiskFilt53D2Coefs[93];
+extern double quiskFilt144D3Coefs[147];
 extern double quiskFilt240D5Coefs[115];
 extern double quiskFilt240D5CoefsSharp[245];
 extern double quiskFilt48dec24Coefs[98];
@@ -76,3 +78,7 @@ extern double quiskFiltTx8kAudioB[168];
 extern double quiskFilt16dec8Coefs[62];
 extern double quiskFilt120s03[480];
 extern double quiskFiltI3D25Coefs[825];
+extern double quiskDgtFilt48Coefs[520];
+extern double quiskFilt300D5Coefs[125];
+extern double quiskFilt300D6Coefs[248];
+extern double quiskFilt240D4Coefs[100];
