@@ -38,7 +38,7 @@ struct freedv_advanced {	// from freedv_api.h
 
 #ifdef MS_WINDOWS
 #include <windows.h>
-HMODULE WINAPI hLib;
+HMODULE hLib;
 #define GET_HANDLE1			hLib = LoadLibrary("libcodec2.dll")
 #define GET_HANDLE2			hLib = LoadLibrary(".\\freedvpkg\\libcodec2.dll")
 #define GET_HANDLE3			hLib = LoadLibrary(".\\freedvpkg\\libcodec2_32.dll")
@@ -530,8 +530,17 @@ static int OpenFreedv(void)	// Called from the GUI thread or sound thread
 			return 0;	// failure
 		}
 	}
+	if (requested_mode == FREEDV_MODE_700E) {
+		if (handle_index >= 3) {	// Quisk provided codec2
+			CloseFreedv();
+			requested_mode = -1;
+			if (DEBUG)
+				printf("freedv_open: Failure because mode 700E requires a system installation of codec2\n");
+			return 0;	// failure
+		}
+	}
 
-   	if (requested_mode == FREEDV_MODE_700D && freedv_open_advanced) {
+   	if ((requested_mode == FREEDV_MODE_700D || requested_mode == FREEDV_MODE_700E) && freedv_open_advanced) {
        	struct freedv_advanced adv;
        	adv.interleave_frames = interleave_frames;
        	hF = freedv_open_advanced(requested_mode, &adv);
@@ -640,6 +649,9 @@ PyObject * quisk_freedv_set_options(PyObject * self, PyObject * args, PyObject *
 	if (mode == -1) {	// mode was not entered
 		;
 	}
+	else if (mode == FREEDV_MODE_700E && freedv_version >= 14) {
+		requested_mode = mode;
+	}
 	else if (mode == FREEDV_MODE_2020) {
 		if (checkAvxSupport() && handle_index <= 2 && freedv_version >= 13)
 			requested_mode = mode;
