@@ -39,6 +39,8 @@ class Hardware(BaseHardware):
     self.antenna_labels = ('Ant 0', 'Ant 1')	# labels for the antenna button
     self.antenna_index = 0			# index of antenna in use
     self.delay_config = 30			# Delay sending message to HL2 until after sound starts
+    self.TFRC_counter = 0		# Call for power etc. at intervals
+    self.key_was_down = 0
     try:
       self.repeater_delay = conf.repeater_delay		# delay for changing repeater frequency in seconds
     except:
@@ -112,7 +114,19 @@ class Hardware(BaseHardware):
           elif bid >= 0 and data[10] != bid:
             pass
           else:
-            st = 'Capture from Hermes device: Mac %2x:%2x:%2x:%2x:%2x:%2x, Code version %d, ID %d' % tuple(data[3:11])
+            if data[10] == 6:	# Hermes Lite
+              num_rx = int(data[0x13])
+              if num_rx < 1:
+                num_rx = 1
+              elif num_rx > 10:
+                num_rx = 10
+            else:
+              num_rx = 1
+            dta = data[3:10]
+            dta.append(data[0x15])
+            dta.append(num_rx)
+            dta.append(data[10])
+            st = 'Capture from Hermes: Mac %2x:%2x:%2x:%2x:%2x:%2x, Code version %d.%d, Rx %d, ID %d' % tuple(dta)
             self.hermes_mac = data[3:9]
             self.hermes_ip = addr[0]
             self.hermes_code_version = data[9]
@@ -328,9 +342,17 @@ class Hardware(BaseHardware):
   def ReturnFrequency(self):	# Return the current tuning and VFO frequency
     return None, None			# frequencies have not changed
   def HeartBeat(self):
-    self.hermes_temperature, self.hermes_fwd_power, self.hermes_rev_power, self.hermes_pa_current = QS.get_hermes_TFRC()
-    if self.application.bottom_widgets:
-      self.application.bottom_widgets.UpdateText()
+    self.TFRC_counter += 1
+    key_down = QS.is_key_down()
+    if key_down and not self.key_was_down:	# reset on key down
+      self.TFRC_counter = 0
+      QS.get_hermes_TFRC()
+    self.key_was_down = key_down
+    if self.TFRC_counter >= 3:
+      self.TFRC_counter = 0
+      self.hermes_temperature, self.hermes_fwd_power, self.hermes_rev_power, self.hermes_pa_current, self.hermes_fwd_peak, self.hermes_rev_peak = QS.get_hermes_TFRC()
+      if self.application.bottom_widgets:
+        self.application.bottom_widgets.UpdateText()
     if self.delay_config > 0:
       self.delay_config -= 1
       if self.delay_config == 0:	# Delay sending these config parameters
@@ -362,6 +384,8 @@ class Hardware(BaseHardware):
     self.antenna_index = btn.index
     self.ChangeBandFilters()
   def ChangeBandFilters(self):
+    if not hasattr(self.application, "multi_rx_screen"):
+      return	# Needed for the VNA program
     self.SetControlBit(0x00, 13, self.antenna_index)
     highest = self.band
     freq = self.conf.BandEdge.get(highest, (0, 0))[0]
@@ -397,12 +421,18 @@ class Hardware(BaseHardware):
     self.SetTxLevel()
   def OnSpot(self, level):
     # level is -1 for Spot button Off; else the Spot level 0 to 1000.
-    if level >= 0 and self.conf.hermes_antenna_tuner == "Tune":
-      self.SetControlBit(0x09, 20, 1)
-      if DEBUG: print("OnSpot antenna tuner: On")
-    else:
+    if level < 0 or self.conf.hermes_antenna_tuner != "Tune":
+      self.SetControlBit(0x09, 17, 0)
       self.SetControlBit(0x09, 20, 0)
       if DEBUG: print("OnSpot antenna tuner: Off")
+    elif level == 0:
+      self.SetControlBit(0x09, 17, 1)		# Bypass tuner if bit 17 set
+      self.SetControlBit(0x09, 20, 1)
+      if DEBUG: print("OnSpot antenna tuner: Bypass")
+    else:
+      self.SetControlBit(0x09, 17, 0)
+      self.SetControlBit(0x09, 20, 1)
+      if DEBUG: print("OnSpot antenna tuner: Tune")
   def VarDecimGetChoices(self):		# return text labels for the control
     return self.var_rates
   def VarDecimGetLabel(self):		# return a text label for the control
@@ -492,6 +522,7 @@ class Hardware(BaseHardware):
     QS.pc_to_hermes(self.pc2hermes)
   def SetVNA(self, key_down=None, vna_start=None, vna_stop=None, vna_count=None, do_tx=False):
     if vna_count is not None:	# must be called first
+      if DEBUG: print("vna_count", vna_count)
       self.vna_count = vna_count
     if vna_start is None:
       start = 0
@@ -526,9 +557,12 @@ class Hardware(BaseHardware):
       if not self.vna_started:
         self.vna_started = True
         self.SetControlByte(9, 2, 0x80)		# turn on VNA mode
-      QS.set_PTT(1)
+        if DEBUG: print("vna_started TRUE")
+      QS.set_key_down(1)
+      if DEBUG: print ("vna key down")
     else:
-      QS.set_PTT(0)
+      QS.set_key_down(0)
+      if DEBUG: print ("vna key up")
     return start, stop	# Return actual frequencies after all phase rounding
   def ImmediateChange(self, name):
     if name == 'keyupDelay':
