@@ -11,7 +11,7 @@
 #include "freedv.h"
 
 #ifdef MS_WINDOWS
-#include <Winsock2.h>
+#include <winsock2.h>
 static int mic_cleanup = 0;		// must clean up winsock
 #else
 #include <sys/socket.h>
@@ -909,16 +909,6 @@ void quisk_hermes_tx_send(int tx_socket, int * tx_records)
 		last_play_state = quisk_play_state;
 		serial_key_samples(NULL, 0);
 	}
-	if (quisk_play_state == SOFTWARE_CWKEY) {
-		serial_key_samples(cw_samples, 63 * 2);
-		j = 0;
-		for (i = 0; i < 63 * 2; i++) {
-			hermes_buf[j++] = (short)cimag(cw_samples[i]);
-			hermes_buf[j++] = (short)creal(cw_samples[i]);
-		}
-		hermes_read_index = 0;
-		hermes_num_samples = 63 * 2;
-	}
 	if (quisk_is_key_down()) {
 		if (quisk_play_state == HARDWARE_CWKEY)
 			mox_bit = 0;
@@ -934,6 +924,16 @@ void quisk_hermes_tx_send(int tx_socket, int * tx_records)
 	if (*tx_records / ratio < 63 * 2)		// tx_records is the number of samples received for each receiver
 		return;
 	// Send 63*2 Tx samples with control bytes
+	if (quisk_play_state == SOFTWARE_CWKEY) {	// Send CW samples
+		serial_key_samples(cw_samples, 63 * 2);
+		j = 0;
+		for (i = 0; i < 63 * 2; i++) {
+			hermes_buf[j++] = (short)cimag(cw_samples[i]);
+			hermes_buf[j++] = (short)creal(cw_samples[i]);
+		}
+		hermes_read_index = 0;
+		hermes_num_samples = 63 * 2;
+	}
 	//printf ("Buffer usage %.1f %%\n", 100.0 * hermes_num_samples / HERMES_TX_BUF_SAMPLES);
 	//printf ("Tx  quisk_hermes_tx_send ratio %d, count %d, samples %d\n", ratio, *tx_records, hermes_num_samples);
 	*tx_records -= 63 * 2 * ratio;
@@ -1258,7 +1258,7 @@ int quisk_process_microphone(int mic_sample_rate, complex double * cSamples, int
 			cSamples[i] = 0;
 	}
 	if (key_down != key_was_down) {	
-		key_down_counter = 4800;	// Key was pressed. Zero the first few samples to clear the buffers.
+		key_down_counter = quisk_start_ssb_delay * 48;	// Key was pressed. Zero the first few samples to clear the buffers.
 		init_alc(&tx_alc, 0);		// init ALC for Tx and RECORD_MIC
 		key_was_down = key_down;
 	}
@@ -1579,11 +1579,10 @@ void quisk_set_tx_mode(void)	// called when the mode rxMode is changed
 }
 
 #define CW_RISE_MSEC	5
-#define CW_DELAY_MSEC	15
-#define CW_DELAY_SAMPLES	(CW_DELAY_MSEC * 48)
+#define CW_DELAY_SAMPLES	(START_CW_DELAY_MAX * 48)
 static void serial_key_samples(complex double * cSamples, int count)	// called from the sound thread
 {  // Internal CW from serial port key - fill cSamples with CW samples
-	int i, key_down;
+	int i, key_down, delay;
 	static double ampl = 0;
 	static char delay_line[CW_DELAY_SAMPLES];	// play CW a fixed delay after the key
 	static int delay_index;
@@ -1597,11 +1596,13 @@ static void serial_key_samples(complex double * cSamples, int count)	// called f
 		return;
 	}
 
+	delay = quisk_start_cw_delay * 48;
 	for (i = 0; i < count; i++) {
 		key_down = delay_line[delay_index];
 		delay_line[delay_index] =  QUISK_CWKEY_DOWN;
-		if (++delay_index >= CW_DELAY_SAMPLES)
+		if (++delay_index >= delay) {
 			delay_index = 0;
+		}
 		if (key_down) {		// key is down
 			if (ampl < themax) {
 				ampl += delta;
