@@ -5,20 +5,22 @@
 // Sound parameters
 //
 #define QUISK_SC_SIZE		256
-#define QUISK_PATH_SIZE     256         // max file path length
-#define IP_SIZE				32
+#define QUISK_PATH_SIZE		256		// max file path length
+#define IP_SIZE			32
 #define MAX_FILTER_SIZE		10001
-#define BIG_VOLUME			2.2e9
-#define CLOSED_TEXT			"The sound device is closed."
-#define CLIP32				2147483647
-#define CLIP16				32767
+#define BIG_VOLUME		2.2e9
+#define CLOSED_TEXT		"The sound device is closed."
+#define CLIP32			2147483647
+#define CLIP16			32767
 #define SAMP_BUFFER_SIZE	66000		// size of arrays used to capture samples
-#define IMD_TONE_1			1200		// frequency of IMD test tones
-#define IMD_TONE_2			1600
-#define INTERP_FILTER_TAPS	85			// interpolation filter
+#define IMD_TONE_1		1200		// frequency of IMD test tones
+#define IMD_TONE_2		1600
+#define INTERP_FILTER_TAPS	85		// interpolation filter
 #define MIC_OUT_RATE		48000		// mic post-processing sample rate
-#define PA_LIST_SIZE 		16			// max number of pulseaudio devices
-#define QUISK_MAX_RECEIVERS	9			// max number of receiver channels
+#define PA_LIST_SIZE		16		// max number of pulseaudio devices
+#define QUISK_MAX_SUB_RECEIVERS		9	// Maximum number of sub-receiver channels in addition to the main receiver
+#define QUISK_INDEX_SUB_RX1		4	// Index of sub-receiver Rx1 in quiskPlaybackDevices
+#define START_CW_DELAY_MAX	250		// Maximum delay for start_cw_delay
 
 // Test the audio: 0 == No test; normal operation;
 // 1 == Copy real data to the output; 2 == copy imaginary data to the output;
@@ -138,7 +140,8 @@ struct sound_dev {				// data for sound capture or playback device
    // frames.
 	int read_frames;
 	int latency_frames;			// desired latency in audio play samples
-	int play_buf_size;			// size of the sound card playback buffer in frames or bytes
+	int play_buf_size;			// size of the sound card playback buffer in frames (?? or bytes)
+	int play_buf_bytes;			// size of the sound card playback buffer in bytes
         int old_key;                            // previous key up/down state
 	int use_float;				// DirectX: Use IEEE floating point
 	int dataPos;				// DirectX: data position
@@ -286,6 +289,7 @@ extern int quisk_is_vna;			// is this the VNA program?
 extern int quisk_serial_key_errors;		// Error count for the Quisk internal serial key
 extern double quisk_sidetoneVolume;		// Audio output level of the CW sidetone, 0.0 to 1.0
 extern int quisk_serial_key_down;		// The state of the serial port CW key
+extern int quisk_serial_ptt;			// The state of the serial port PTT
 extern int quisk_hardware_cwkey;		// The state of the hardware CW key from UDP or USB
 extern int quisk_midi_cwkey;			// The state of the MIDI CW key
 extern int quisk_sidetoneFreq;			// Frequency in hertz for the sidetone
@@ -296,6 +300,9 @@ extern play_state_t quisk_play_state;		// startup, receiving, sidetone
 extern int freedv_current_mode;			// current FreeDV mode; 700D, @)@) etc.
 extern int n_modem_sample_rate;			// Receive data, decimate to modem_sample_rate, FreeDV codec output data at speech_sample_rate
 extern int n_speech_sample_rate;		// Microphone decimate to speech_sample_rate, Freedv codec output data at modem_sample_rate, interpolate to 48000
+extern int quisk_start_cw_delay;		// milliseconds to delay output on serial or MIDI CW key down
+extern int quisk_start_ssb_delay;		// milliseconds to discard output for all modes except CW
+extern struct sound_dev * quiskPlaybackDevices[];	// array of Playback sound devices
 
 extern PyObject * quisk_set_spot_level(PyObject * , PyObject *);
 extern PyObject * quisk_get_tx_filter(PyObject * , PyObject *);
@@ -328,12 +335,17 @@ extern PyObject * quisk_freedv_get_rx_char(PyObject *, PyObject *);
 extern PyObject * quisk_freedv_set_options(PyObject *, PyObject *, PyObject *);
 extern PyObject * quisk_set_sparams(PyObject *, PyObject *, PyObject *);
 extern PyObject * quisk_freedv_set_squelch_en(PyObject *, PyObject *);
-extern PyObject * quisk_open_key(PyObject *, PyObject *);
+extern PyObject * quisk_open_key(PyObject *, PyObject *, PyObject *);
 extern PyObject * quisk_close_key(PyObject *, PyObject *);
 extern PyObject * quisk_set_sound_name(PyObject *, PyObject *);
 extern PyObject * quisk_alsa_control_midi(PyObject *, PyObject *, PyObject *);
 extern PyObject * quisk_wasapi_control_midi(PyObject *, PyObject *, PyObject *);
 
+// WDSP interface
+#define QUISK_WDSP_RX	1
+extern PyObject * quisk_wdsp_set_parameter(PyObject *, PyObject *, PyObject *);
+extern int wdspFexchange0(int, double *, int);
+
 // These function pointers are the Start/Stop/Read interface for
 // the SDR-IQ and any other C-language extension modules that return
 // radio data samples.
