@@ -222,6 +222,8 @@ class SliderBoxH:
   """A horizontal control with a slider and text with a value.  The text must have a %d or %f if display is True."""
   def __init__(self, parent, text, init, themin, themax, handler, display, pos, width, scale=1):
     self.text = text
+    self.themin = themin
+    self.themax = themax
     self.handler = handler
     self.display = display
     self.scale = scale
@@ -270,6 +272,18 @@ class SliderBoxH:
     # Set slider visual position; does not call handler
     self.slider.SetValue(value)
     self.OnScroll()
+  def GetDecValue(self):	# Return the value 0.0 to 1.0
+    return float(self.slider.GetValue() - self.themin) / (self.themax - self.themin)
+  def SetDecValue(self, value, do_cmd=True):	# Set the value with a decimal 0.0 to 1.0
+    if value < 0.0:
+      value = 0.0
+    elif value > 1.0:
+      value = 1.0
+    value = value * (self.themax - self.themin) + self.themin
+    value = int(value + 0.5)
+    self.SetValue(value)
+    if do_cmd:
+      self.handler(None)
 
 class SliderBoxHH(SliderBoxH, wx.BoxSizer):
   """A horizontal control with a slider and text with a value.  The text must have a %d if display is True."""
@@ -289,6 +303,7 @@ class SliderBoxV(wx.BoxSizer):
     wx.BoxSizer.__init__(self, wx.VERTICAL)
     self.slider = wx.Slider(parent, -1, init, themin, themax, style=wx.SL_VERTICAL|wx.SL_INVERSE)
     self.slider.Bind(wx.EVT_SCROLL, handler)
+    self.handler = handler
     sw, sh = self.slider.GetSize()
     self.text = text
     self.themin = themin
@@ -333,6 +348,18 @@ class SliderBoxV(wx.BoxSizer):
   def SetValue(self, value):
     # Set slider visual position; does not call handler
     self.slider.SetValue(value)
+  def GetDecValue(self):	# Return the value 0.0 to 1.0
+    return float(self.slider.GetValue() - self.themin) / (self.themax - self.themin)
+  def SetDecValue(self, value, do_cmd=True):	# Set the value with a decimal 0.0 to 1.0
+    if value < 0.0:
+      value = 0.0
+    elif value > 1.0:
+      value = 1.0
+    value = value * (self.themax - self.themin) + self.themin
+    value = int(value + 0.5)
+    self.SetValue(value)
+    if do_cmd:
+      self.handler(None)
 
 class QuiskText1(wx.lib.stattext.GenStaticText):
   # Self-drawn text for QuiskText.
@@ -412,7 +439,8 @@ class QuiskText(wx.BoxSizer):
 
 class QuiskButtons:
   """Base class for special buttons."""
-  def __init__(self):
+  def __init__(self, idName):
+    self.idName = idName	# idName can be ''
     self.up_brush = wx.Brush(conf.color_btn)
     r, g, b = self.up_brush.GetColour().Get(False)
     r, g, b = min(255,r+32), min(255,g+32), min(255,b+32)
@@ -504,6 +532,7 @@ class QuiskButtons:
 
 class QuiskBitmapButton(wx.lib.buttons.GenBitmapButton):
   def __init__(self, parent, command, bitmap, use_right=False):
+    self.idName = "Bitmap"
     self.command = command
     self.bitmap = bitmap
     wx.lib.buttons.GenBitmapButton.__init__(self, parent, -1, bitmap)
@@ -533,7 +562,7 @@ class QuiskBitmapButton(wx.lib.buttons.GenBitmapButton):
 class QuiskPushbutton(QuiskButtons, wx.lib.buttons.GenButton):
   """A plain push button widget."""
   def __init__(self, parent, command, text, use_right=False, text_color=None, style=0):
-    QuiskButtons.__init__(self)
+    QuiskButtons.__init__(self, text)
     wx.lib.buttons.GenButton.__init__(self, parent, -1, text, style=style)
     self.command = command
     self.Bind(wx.EVT_BUTTON, self.OnButton)
@@ -556,7 +585,7 @@ class QuiskPushbutton(QuiskButtons, wx.lib.buttons.GenButton):
 class QuiskRepeatbutton(QuiskButtons, wx.lib.buttons.GenButton):
   """A push button that repeats when held down."""
   def __init__(self, parent, command, text, up_command=None, use_right=False):
-    QuiskButtons.__init__(self)
+    QuiskButtons.__init__(self, text)
     wx.lib.buttons.GenButton.__init__(self, parent, -1, text)
     self.command = command
     self.up_command = up_command
@@ -606,12 +635,25 @@ class QuiskRepeatbutton(QuiskButtons, wx.lib.buttons.GenButton):
       self.SendCommand(self.command)
   def OnButton(self, event):
     pass	# button command not used
+  def Shortcut(self, event, button_name=None):
+    if not self.IsEnabled():
+      return
+    if button_name == "_end_":
+      self.repeat_state = 0
+      self.timer.Stop()
+      self.SendCommand(self.up_command)
+    else:
+      self.shift = False
+      self.control = False
+      self.SendCommand(self.command)
+      self.repeat_state = 1		# first button push
+      self.timer.Start(milliseconds=300, oneShot=True)
 
 class QuiskCheckbutton(QuiskButtons, wx.lib.buttons.GenToggleButton):
   """A button that pops up and down, and changes color with each push."""
   # Check button; get the checked state with self.GetValue()
   def __init__(self, parent, command, text, color=None, use_right=False):
-    QuiskButtons.__init__(self)
+    QuiskButtons.__init__(self, text)
     wx.lib.buttons.GenToggleButton.__init__(self, parent, -1, text)
     self.InitButtons(text)
     self.Bind(wx.EVT_BUTTON, self.OnButton)
@@ -641,8 +683,9 @@ class QuiskCheckbutton(QuiskButtons, wx.lib.buttons.GenToggleButton):
   def OnRightUp(self, event):
     self.OnLeftUp(event)
     self.direction = 1
-  def Shortcut(self, event):
-    self.SetValue(not self.GetValue(), True)
+  def Shortcut(self, event, button_name=None):
+    if self.IsEnabled():
+      self.SetValue(not self.GetValue(), True)
 
 class QuiskBitField(wx.Window):
   """A control used to set/unset bits."""
@@ -1074,11 +1117,16 @@ class QuiskCycleCheckbutton(QuiskCheckbutton):
   def DrawLabel(self, dc, width, height, dx=0, dy=0):
     QuiskCheckbutton.DrawLabel(self, dc, width, height, dx, dy)
     self.DrawGlyphCycle(dc, width, height)
-  def Shortcut(self, event):
-    index = self.index + 1
-    if index >= len(self.labels):
-      index = 0
-    self.SetIndex(index, True)
+  def Shortcut(self, event, button_name=None):
+    if self.IsEnabled():
+      if not self.is_radio or self.button_down:
+        self.direction = 1
+        index = self.index + 1
+        if index >= len(self.labels):
+          index = 0
+        self.SetIndex(index, True)
+      else:
+        self.SetIndex(self.index, True)
 
 class RadioButtonGroup:
   """This class encapsulates a group of radio buttons.  This class is not a button!
@@ -1089,7 +1137,7 @@ class RadioButtonGroup:
   """
   def __init__(self, parent, command, labels, default, shortcuts=()):
     self.command = command
-    self.buttons = []
+    self.buttons = []	# This list of buttons can also contain WrapControls
     self.button = None
     self.shortcuts = list(shortcuts[:])
     self.last_shortcut = 0
@@ -1127,7 +1175,7 @@ class RadioButtonGroup:
     for b in self.buttons:
       if self.button is not None:
         b.SetValue(False)
-      elif isinstance(b, QuiskCycleCheckbutton):
+      elif isinstance(b, QuiskCycleCheckbutton) or (isinstance(b, WrapControl) and isinstance(b.button, QuiskCycleCheckbutton)):
         try:
           index = b.labels.index(label)
         except ValueError:
@@ -1138,7 +1186,10 @@ class RadioButtonGroup:
           self.button = b
           b.SetValue(True)
           if direction is not None:
-            b.direction = direction
+            if isinstance(b, WrapControl):
+              b.button.direction = direction
+            else:
+              b.direction = direction
       elif b.GetLabel() == label:
         b.SetValue(True)
         self.button = b
@@ -1166,11 +1217,11 @@ class RadioButtonGroup:
     return self.button.GetLabel()
   def GetSelectedButton(self):		# return the selected button
     return self.button
-  def GetIndex(self):	# Careful.  Some buttons are complex.
+  def GetIndex(self):	# Careful.  Some buttons are WrapControls.
     if not self.button:
       return None
     return self.buttons.index(self.button)
-  def Shortcut(self, event):
+  def Shortcut(self, event):	# Not used for Midi
     # Multiple buttons can have the same shortcut, so move to the next one.
     index = self.last_shortcut + 1
     length = len(self.shortcuts)
@@ -1324,6 +1375,8 @@ class RadioBtnPopup:
       event.SetEventObject(self.first_button)
       self.pop_command(event)
     self.first_button.direction = direc
+  def GetButtons(self):
+    return self.RbDialog.RbGroup.GetButtons()
   def Refresh(self):
     pass
   def ChangeSlider(self, slider_value):
@@ -1340,6 +1393,8 @@ class RadioBtnPopup:
     return self.parent
   def GetSize(self):
     return self.first_button.GetSize()
+  def Shortcut(self, event, button_name=None):
+    self.SetLabel(button_name, True)
 
 class FreqSetter(wx.TextCtrl):
   def __init__(self, parent, x, y, label, fmin, fmax, freq, command):
