@@ -25,7 +25,7 @@
 #include <mmreg.h>
 #include <functiondiscoverykeys_devpkey.h>
 #include <avrt.h>
-#include <Ksmedia.h>
+#include <ksmedia.h>
 #include <processthreadsapi.h>
 
 #define EXIT_ON_ERROR(hres)  \
@@ -59,6 +59,7 @@ struct dev_data_t {
 	IAudioCaptureClient *pCaptureClient;
 	IAudioRenderClient *pRenderClient;
 	UINT32 bufferSizeFrames;	// size of the Wasapi buffer in frames
+	AUDCLNT_SHAREMODE sharemode;
 	HANDLE hEvent;
 	int playbuf_underrun_reset;		// playbuf is used for the play device threadproc
 	int playbuf_underrun_msg;
@@ -71,11 +72,30 @@ struct dev_data_t {
 
 static LPWSTR to_pwsz(const char *str) {	// Convert UTF-8 string to Wide Character string.
 	static wchar_t wchar_buffer[256];
-	if (MultiByteToWideChar(CP_UTF8, MB_PRECOMPOSED, str, -1, wchar_buffer, 256) == 0)
+	if (MultiByteToWideChar(CP_UTF8, 0, str, -1, wchar_buffer, 256) == 0)
 		return NULL;
 	return wchar_buffer;
 }
 
+static void quisk_reset_audio_device(struct sound_dev * dev)
+{  // Contributed by Ben Cahill, AC2YD, February 2021
+	struct dev_data_t * DD = dev->device_data;
+	HRESULT hr;
+
+	hr = IAudioClient_Stop(DD->pAudioClient);
+	if (hr != S_OK) {
+		QuiskPrintf("%s:  IAudioClient_Stop() returned hr = 0x%lx!!\n", dev->stream_description, hr);
+	}
+	hr = IAudioClient_Reset(DD->pAudioClient);
+	if (hr != S_OK) {
+		QuiskPrintf("%s:  IAudioClient_Reset() returned hr = 0x%lx\n!!", dev->stream_description, hr);
+	}
+	hr = IAudioClient_Start(DD->pAudioClient);
+	if (hr != S_OK) {
+		QuiskPrintf("%s:  IAudioClient_Start() returned hr = 0x%lx!!", dev->stream_description, hr);
+	}
+}
+
 static void close_device(struct sound_dev *, const char *);
 static void open_wasapi_playback(struct sound_dev *, int);
 
@@ -87,10 +107,11 @@ static DWORD WINAPI playdevice_threadproc(LPVOID lpParameter)	// Called by a spe
 	BYTE * pData;
 	BYTE * ptSamples;
         int64_t nRead, nWrite;
-	int i, two, ch_I, ch_Q, frames_in_buffer, bytes_per_sample, bytes_per_frame, index;
+	int i, two, ch_I, ch_Q, frames_in_buffer, bytes_per_sample, bytes_per_frame, index, frames_to_end;
 	DWORD taskIndex = 0;
 	HANDLE hTask;
-
+	UINT32 write_frames;
+	UINT32 NumPaddingFrames;
 	if (CoInitializeEx(NULL, COINIT_APARTMENTTHREADED) != S_OK)
 		QuiskPrintf("%s:  CoInitializeEx failed\n", dev->stream_description);
 	hTask = AvSetMmThreadCharacteristics(TEXT("Pro Audio"), &taskIndex);
@@ -100,9 +121,11 @@ static DWORD WINAPI playdevice_threadproc(LPVOID lpParameter)	// Called by a spe
 		close_device(dev, "Could not create playback client");
 		return 1;
 	}
-	// Load the buffer with silence before starting the stream.
-	if (SUCCEEDED(IAudioRenderClient_GetBuffer(DD->pRenderClient, DD->bufferSizeFrames, &pData)))
-		IAudioRenderClient_ReleaseBuffer(DD->pRenderClient, DD->bufferSizeFrames, AUDCLNT_BUFFERFLAGS_SILENT);
+	if (DD->sharemode == AUDCLNT_SHAREMODE_EXCLUSIVE) {
+		// Load the buffer with silence before starting the stream.
+		if (SUCCEEDED(IAudioRenderClient_GetBuffer(DD->pRenderClient, DD->bufferSizeFrames, &pData)))
+			IAudioRenderClient_ReleaseBuffer(DD->pRenderClient, DD->bufferSizeFrames, AUDCLNT_BUFFERFLAGS_SILENT);
+	}
 	dev->handle = DD->pRenderClient;
 	if (FAILED(IAudioClient_Start(DD->pAudioClient))) {
 		close_device(dev, "Could not start");
@@ -115,14 +138,27 @@ static DWORD WINAPI playdevice_threadproc(LPVOID lpParameter)	// Called by a spe
 			break;
 		if (DD == NULL || dev->handle == NULL)
 			break;
-		if (IAudioRenderClient_GetBuffer(DD->pRenderClient, DD->bufferSizeFrames, &pData) != S_OK) {
+		if (DD->sharemode == AUDCLNT_SHAREMODE_EXCLUSIVE) {
+			write_frames = DD->bufferSizeFrames;
+		}
+		else {
+			if (IAudioClient_GetCurrentPadding(DD->pAudioClient, &NumPaddingFrames) != S_OK) {
+				QuiskPrintf("%s:  playdevice: GetCurrentPadding failed\n", dev->stream_description);
+		        	dev->dev_error++;
+				continue;
+			}
+			write_frames = DD->bufferSizeFrames - NumPaddingFrames;
+			if (write_frames == 0)
+				continue;
+		}
+		if (IAudioRenderClient_GetBuffer(DD->pRenderClient, write_frames, &pData) != S_OK) {
 			QuiskPrintf("%s:  playdevice: GetBuffer failed\n", dev->stream_description);
 		        dev->dev_error++;
 			continue;
 		}
 		if (quisk_play_state < RECEIVE) {       // Shutdown or Starting
 			DD->playbuf_underrun_reset = 0;
-			IAudioRenderClient_ReleaseBuffer(DD->pRenderClient, DD->bufferSizeFrames, AUDCLNT_BUFFERFLAGS_SILENT);
+			IAudioRenderClient_ReleaseBuffer(DD->pRenderClient, write_frames, AUDCLNT_BUFFERFLAGS_SILENT);
 			continue;
 		}
 		nRead = atomic_load(&DD->playbuf_nRead);
@@ -133,15 +169,15 @@ static DWORD WINAPI playdevice_threadproc(LPVOID lpParameter)	// Called by a spe
 				DD->playbuf_underrun_reset = 0;
 			}
 			else {
-				IAudioRenderClient_ReleaseBuffer(DD->pRenderClient, DD->bufferSizeFrames, AUDCLNT_BUFFERFLAGS_SILENT);
+				IAudioRenderClient_ReleaseBuffer(DD->pRenderClient, write_frames, AUDCLNT_BUFFERFLAGS_SILENT);
 				continue;
 			}
 		}
-		else if (frames_in_buffer < DD->bufferSizeFrames) {
+		else if (frames_in_buffer < write_frames) {
 			DD->playbuf_underrun_reset = 1;
 			DD->playbuf_underrun_msg = 1;
 			dev->dev_underrun++;
-			IAudioRenderClient_ReleaseBuffer(DD->pRenderClient, DD->bufferSizeFrames, AUDCLNT_BUFFERFLAGS_SILENT);
+			IAudioRenderClient_ReleaseBuffer(DD->pRenderClient, write_frames, AUDCLNT_BUFFERFLAGS_SILENT);
 			continue;
 		}
 		two = dev->num_channels >= 2;
@@ -150,7 +186,7 @@ static DWORD WINAPI playdevice_threadproc(LPVOID lpParameter)	// Called by a spe
 		bytes_per_sample = dev->sample_bytes;
 		bytes_per_frame = bytes_per_sample * dev->num_channels;
 		if ((rxMode == CWU || rxMode == CWL) && quiskSpotLevel < 0 && dev->dev_index == t_MicPlayback) {	// SoftRock Tx CW from wasapi
-			for (i = 0; i < DD->bufferSizeFrames; i++) {
+			for (i = 0; i < write_frames; i++) {
 				ptSamples = quisk_make_txIQ(dev, 0);
 				memcpy(pData + ch_I * bytes_per_sample, ptSamples, bytes_per_sample);
 				if (two) {
@@ -161,7 +197,7 @@ static DWORD WINAPI playdevice_threadproc(LPVOID lpParameter)	// Called by a spe
 			}
 		}
 		else if (quisk_play_state > RECEIVE && quisk_active_sidetone == 1 && dev->dev_index == t_Playback) {	// Sidetone from wasapi
-			for (i = 0; i < DD->bufferSizeFrames; i++) {
+			for (i = 0; i < write_frames; i++) {
 				ptSamples = quisk_make_sidetone(dev, 0);
 				memcpy(pData + ch_I * bytes_per_sample, ptSamples, bytes_per_sample);
 				if (two)
@@ -171,11 +207,19 @@ static DWORD WINAPI playdevice_threadproc(LPVOID lpParameter)	// Called by a spe
 		}
 		else {		// copy sound samples from the buffer to the sound device.
                         index = nRead % DD->playbuf_nFrames;
+			frames_to_end = DD->playbuf_nFrames - index;
 			ptSamples = DD->playbuf_buf + index * bytes_per_frame;
-			memcpy(pData, ptSamples, DD->bufferSizeFrames * bytes_per_frame);
+			if (write_frames <= frames_to_end) {
+				memcpy(pData, ptSamples, write_frames * bytes_per_frame);
+			}
+			else {
+				memcpy(pData, ptSamples, frames_to_end * bytes_per_frame);
+				memcpy(pData + frames_to_end * bytes_per_frame,
+					DD->playbuf_buf, (write_frames - frames_to_end) * bytes_per_frame);
+			}
 		}
-		IAudioRenderClient_ReleaseBuffer(DD->pRenderClient, DD->bufferSizeFrames, 0);
-		nRead = nRead + DD->bufferSizeFrames;	// update number of frames read
+		IAudioRenderClient_ReleaseBuffer(DD->pRenderClient, write_frames, 0);
+		nRead = nRead + write_frames;	// update number of frames read
 		atomic_store(&DD->playbuf_nRead, nRead);
 	}
 	AvRevertMmThreadCharacteristics(hTask);
@@ -228,13 +272,17 @@ void quisk_write_wasapi(struct sound_dev * dev, int nSamples, complex double * c
 		}
 	}
 	if (frames_in_buffer + nSamples >= DD->playbuf_nFrames) {
-		if (quisk_sound_state.verbose_sound)
-			QuiskPrintf("%s:  play_buffer overflow frames_in_buffer %d, nSamples %d\n", dev->stream_description,
-					frames_in_buffer, nSamples);
 		dev->dev_error++;
 		nSamples = DD->playbuf_nFrames * 9 / 10 - frames_in_buffer;     // almost fill buffer
-		if (nSamples < 0)      // buffer is nearly full
+		if (nSamples < 0) {      // buffer is nearly full
 			DD->playbuf_overflow_reset = 1;
+			quisk_reset_audio_device(dev);	// Ben Cahill: Correct fault in the play callback thread
+			if (quisk_sound_state.verbose_sound)
+				QuiskPrintf("%s:  play_buffer overflow and reset, frames_in_buffer %d, nSamples %d\n", dev->stream_description, frames_in_buffer, nSamples);
+		}
+		else if (quisk_sound_state.verbose_sound) {
+			QuiskPrintf("%s:  play_buffer overflow, frames_in_buffer %d, nSamples %d\n", dev->stream_description, frames_in_buffer, nSamples);
+		}
 	}
 	if (nSamples <= 0)
 		return;
@@ -300,14 +348,16 @@ int quisk_read_wasapi(struct sound_dev * dev, complex double * cSamples)
 {	// Called from Quisk by the sound thread. Read samples from the sound card capture buffer.
 	// cSamples can be NULL to discard samples.
 	struct dev_data_t * DD = dev->device_data;
-	int i;
+	int i, bytes_per_frame, num_channels, ch_I, ch_Q;
 	BYTE * pData;
 	UINT32 numFramesAvailable;
 	DWORD flags;
 	double samp_r, samp_i;
+	int8_t  * ptInt8;
 	int16_t * ptInt16;
 	int32_t * ptInt32;
 	float   * ptFloat32;
+	int32_t iReal, iImag;
 	int nSamples;
 	HRESULT hr;
 
@@ -315,6 +365,10 @@ int quisk_read_wasapi(struct sound_dev * dev, complex double * cSamples)
 		return 0;
 
 	nSamples = 0;
+	ch_I = dev->channel_I;
+	ch_Q = dev->channel_Q;
+	bytes_per_frame = dev->sample_bytes * dev->num_channels;
+        num_channels = dev->num_channels;
 	while (1) {	// Get the available data
 		hr = IAudioCaptureClient_GetBuffer(DD->pCaptureClient, &pData, &numFramesAvailable, &flags, NULL, NULL);
 		if (hr == AUDCLNT_S_BUFFER_EMPTY) {
@@ -335,27 +389,39 @@ int quisk_read_wasapi(struct sound_dev * dev, complex double * cSamples)
 			case Int16:
 				ptInt16 = (int16_t *)pData;
 				for (i = 0; i < numFramesAvailable; i++) {
-					samp_r = *ptInt16++;
-					samp_i = *ptInt16++;
+					samp_r = ptInt16[ch_I];
+					samp_i = ptInt16[ch_Q];
 					cSamples[nSamples++] = (samp_r + I * samp_i) * CLIP16;
+                                        ptInt16 += num_channels;
 				}
 				break;
-			case Int24:
+	                case Int24:     // only works for little-endian
+				ptInt8 = (int8_t *)pData;
+				for (i = 0; i < numFramesAvailable; i++) {
+                                        iReal = 0;
+			                memcpy((int8_t *)&iReal + 1, ptInt8 + ch_I * 3, 3);
+                                        iImag = 0;
+			                memcpy((int8_t *)&iImag + 1, ptInt8 + ch_Q * 3, 3);
+					cSamples[nSamples++] = (iReal + I * iImag);
+                                        ptInt8 += bytes_per_frame;
+				}
 				break;
 			case Int32:
 				ptInt32 = (int32_t *)pData;
 				for (i = 0; i < numFramesAvailable; i++) {
-					samp_r = *ptInt32++;
-					samp_i = *ptInt32++;
+					samp_r = ptInt32[ch_I];
+					samp_i = ptInt32[ch_Q];
 					cSamples[nSamples++] = (samp_r + I * samp_i);
+                                        ptInt32 += num_channels;
 				}
 				break;
 			case Float32:
 				ptFloat32 = (float *)pData;
 				for (i = 0; i < numFramesAvailable; i++) {
-					samp_r = *ptFloat32++;
-					samp_i = *ptFloat32++;
+					samp_r = ptFloat32[ch_I];
+					samp_i = ptFloat32[ch_Q];
 					cSamples[nSamples++] = (samp_r + I * samp_i) * CLIP32;
+                                        ptFloat32 += num_channels;
 				}
 				break;
 			}
@@ -415,12 +481,14 @@ void quisk_play_wasapi(struct sound_dev * dev, int nSamples, complex double * cS
 {	// Called from Quisk by the sound thread. Write samples to the sound card buffer.
 	UINT32 numFramesPadding;
 	BYTE *pData;
+	int8_t * ptb;
 	int16_t * pts;
 	int32_t * ptl;
 	float * ptf;
+        int32_t samp32;
 	struct dev_data_t * DD = dev->device_data;
 	double buffer_fill;
-	int n, frames;
+	int n, frames, bytes_per_frame;
 
 	if (DD == NULL || dev->handle == NULL)
 		return;
@@ -508,7 +576,16 @@ void quisk_play_wasapi(struct sound_dev * dev, int nSamples, complex double * cS
 			ptf += dev->num_channels;
 		}
 		break;
-	case Int24:
+        case Int24:     // only works for little-endian
+		ptb = (int8_t *)pData;
+	        bytes_per_frame = dev->sample_bytes * dev->num_channels;
+		for (n = 0; n < nSamples; n++) {
+			samp32 = volume * creal(cSamples[n]);
+			memcpy(ptb + dev->channel_I * 3, (int8_t *)&samp32 + 1, 3);
+			samp32 = volume * cimag(cSamples[n]);
+			memcpy(ptb + dev->channel_Q * 3, (int8_t *)&samp32 + 1, 3);
+			ptb += bytes_per_frame;
+		}
 		break;
 	}
 	IAudioRenderClient_ReleaseBuffer(DD->pRenderClient, nSamples, 0);
@@ -582,23 +659,39 @@ static void MakeWFext(int extensible, sound_format_t sound_format, WORD nChannel
 	pwfex->Format.wBitsPerSample = dev->sample_bytes * 8;
 }
 
-static int make_format(struct sound_dev * dev, WAVEFORMATEXTENSIBLE * pWaveFormat)
+static int make_format(struct sound_dev * dev, WAVEFORMATEXTENSIBLE * pWaveFormat, AUDCLNT_SHAREMODE sharemode)
 {
 	struct dev_data_t * DD = dev->device_data;
 	WAVEFORMATEX * pEx = (WAVEFORMATEX *)pWaveFormat;
+        WAVEFORMATEX * pClosestMatch;
 	WORD nChannels;
 	sound_format_t sound_format;
+        HRESULT result;
 
 	nChannels = dev->num_channels;
-	for (sound_format = 0; sound_format <= Float32; sound_format++) {
+	for (sound_format = 0; sound_format <= Int24; sound_format++) {
 		MakeWFext(1, sound_format, nChannels, dev, pWaveFormat);
-		if (IAudioClient_IsFormatSupported(DD->pAudioClient, AUDCLNT_SHAREMODE_EXCLUSIVE, pEx, NULL) == S_OK)
+		if (sharemode == AUDCLNT_SHAREMODE_EXCLUSIVE)
+		        result = IAudioClient_IsFormatSupported(DD->pAudioClient, sharemode, pEx, NULL);
+                else {
+                        pClosestMatch = NULL;
+		        result = IAudioClient_IsFormatSupported(DD->pAudioClient, sharemode, pEx, &pClosestMatch);
+                        CoTaskMemFree(pClosestMatch);
+                }
+                if (result == S_OK)
 			return 1;
 	}
 	if (dev->dev_index == t_Playback || dev->dev_index == t_MicCapture) {	// mono device is OK for radio sound or mic
-		for (sound_format = 0; sound_format <= Float32; sound_format++) {
+		for (sound_format = 0; sound_format <= Int24; sound_format++) {
 			MakeWFext(1, sound_format, 1, dev, pWaveFormat);
-			if (IAudioClient_IsFormatSupported(DD->pAudioClient, AUDCLNT_SHAREMODE_EXCLUSIVE, pEx, NULL) == S_OK) {
+		        if (sharemode == AUDCLNT_SHAREMODE_EXCLUSIVE)
+		                result = IAudioClient_IsFormatSupported(DD->pAudioClient, sharemode, pEx, NULL);
+                        else {
+                                pClosestMatch = NULL;
+		                result = IAudioClient_IsFormatSupported(DD->pAudioClient, sharemode, pEx, &pClosestMatch);
+                                CoTaskMemFree(pClosestMatch);
+                        }
+                        if (result == S_OK) {
 				dev->num_channels = 1;
 				dev->channel_I = 0;
 				dev->channel_Q = 0;
@@ -608,9 +701,16 @@ static int make_format(struct sound_dev * dev, WAVEFORMATEXTENSIBLE * pWaveForma
 	}
 	// Try to open with more channels
 	for (nChannels = dev->num_channels + 1; nChannels <= 7; nChannels++) {
-		for (sound_format = 0; sound_format <= Float32; sound_format++) {
+		for (sound_format = 0; sound_format <= Int24; sound_format++) {
 			MakeWFext(1, sound_format, nChannels, dev, pWaveFormat);
-			if (IAudioClient_IsFormatSupported(DD->pAudioClient, AUDCLNT_SHAREMODE_EXCLUSIVE, pEx, NULL) == S_OK) {
+		        if (sharemode == AUDCLNT_SHAREMODE_EXCLUSIVE)
+		                result = IAudioClient_IsFormatSupported(DD->pAudioClient, sharemode, pEx, NULL);
+                        else {
+                                pClosestMatch = NULL;
+		                result = IAudioClient_IsFormatSupported(DD->pAudioClient, sharemode, pEx, &pClosestMatch);
+                                CoTaskMemFree(pClosestMatch);
+                        }
+                        if (result == S_OK) {
 				dev->num_channels = nChannels;
 				return 1;
 			}
@@ -654,6 +754,7 @@ static void open_wasapi_capture(struct sound_dev * dev)
 	LPWSTR pwszID;
 	struct dev_data_t * DD = dev->device_data;
 	HRESULT hr;
+	DWORD err_code;
 
 	dev->dev_errmsg[0] = 0;
 	if (quisk_sound_state.verbose_sound)
@@ -663,7 +764,11 @@ static void open_wasapi_capture(struct sound_dev * dev)
 
 	pwszID = to_pwsz(dev->device_name);
 	if (pwszID == NULL) {
-		close_device(dev, "Failure to convert device name");
+		err_code = GetLastError();
+		snprintf(dev->dev_errmsg, QUISK_SC_SIZE, "Failure 0x%lX to convert device name: %.80s", err_code, dev->device_name);
+		if (quisk_sound_state.verbose_sound)
+			QuiskPrintf("%s\n", dev->dev_errmsg);
+		close_device(dev, NULL);
 		return;
 	}
 	if (FAILED(IMMDeviceEnumerator_GetDevice(pEnumerator, pwszID, &DD->pDevice))) {
@@ -682,9 +787,19 @@ static void open_wasapi_capture(struct sound_dev * dev)
 				(int)def_period / REFTIMES_PER_MICROSEC, (int)min_period / REFTIMES_PER_MICROSEC);
 	}
 
-	if (make_format(dev, &wave_format) == 0) {
-		close_device(dev, "Device can not support the sample rate or number of channels.");
-		return;
+	if (make_format(dev, &wave_format, AUDCLNT_SHAREMODE_EXCLUSIVE) == 0) {		// failure
+		if (make_format(dev, &wave_format, AUDCLNT_SHAREMODE_SHARED) == 0) {	// failure
+			close_device(dev, "Device can not support the sample rate or number of channels.");
+			return;
+		}
+		DD->sharemode = AUDCLNT_SHAREMODE_SHARED;
+		if (quisk_sound_state.verbose_sound)
+			QuiskPrintf("  Exclusive mode refused. Open device in Shared mode\n");
+	}
+	else {
+		DD->sharemode = AUDCLNT_SHAREMODE_EXCLUSIVE;
+		if (quisk_sound_state.verbose_sound)
+			QuiskPrintf("  Open device in Exclusive mode\n");
 	}
 
 	if (quisk_sound_state.verbose_sound) {
@@ -694,10 +809,10 @@ static void open_wasapi_capture(struct sound_dev * dev)
 		QuiskPrintf("  Number of channels %d, bytes per sample %d\n", dev->num_channels, dev->sample_bytes);
 	}
 	hnsRequestedDuration = REFTIMES_PER_MILLISEC * quisk_sound_state.latency_millisecs;
-	hr = IAudioClient_Initialize(DD->pAudioClient, AUDCLNT_SHAREMODE_EXCLUSIVE, 0, hnsRequestedDuration, 0, (WAVEFORMATEX *)&wave_format, NULL);
+	hr = IAudioClient_Initialize(DD->pAudioClient, DD->sharemode, 0, hnsRequestedDuration, 0, (WAVEFORMATEX *)&wave_format, NULL);
 	if (FAILED(hr)) {
 		for ( ;hnsRequestedDuration > REFTIMES_PER_MILLISEC * 20; hnsRequestedDuration -= (REFTIMES_PER_MILLISEC * 20)) {
-			hr = IAudioClient_Initialize(DD->pAudioClient, AUDCLNT_SHAREMODE_EXCLUSIVE, 0, hnsRequestedDuration, 0, (WAVEFORMATEX *)&wave_format, NULL);
+			hr = IAudioClient_Initialize(DD->pAudioClient, DD->sharemode, 0, hnsRequestedDuration, 0, (WAVEFORMATEX *)&wave_format, NULL);
 			if (hr == S_OK)
 				break;
 		}
@@ -732,7 +847,7 @@ static void open_wasapi_capture(struct sound_dev * dev)
 
 static void open_wasapi_playback(struct sound_dev * dev, int use_callback)
 {
-	REFERENCE_TIME hnsRequestedDuration = 0;
+	REFERENCE_TIME hnsRequestedDuration;
 	REFERENCE_TIME def_period, min_period;
 	WAVEFORMATEXTENSIBLE wave_format;
         int i;
@@ -742,6 +857,7 @@ static void open_wasapi_playback(struct sound_dev * dev, int use_callback)
 	HRESULT hr = S_OK;
 	HANDLE hThread = NULL;
 	DWORD ThreadID;
+	DWORD err_code;
 
 	dev->dev_errmsg[0] = 0;
 	if (quisk_sound_state.verbose_sound) {
@@ -752,7 +868,11 @@ static void open_wasapi_playback(struct sound_dev * dev, int use_callback)
 
 	pwszID = to_pwsz(dev->device_name);
 	if (pwszID == NULL) {
-		close_device(dev, "Failure to convert device name");
+		err_code = GetLastError();
+		snprintf(dev->dev_errmsg, QUISK_SC_SIZE, "Failure 0x%lX to convert device name: %.80s", err_code, dev->device_name);
+		if (quisk_sound_state.verbose_sound)
+			QuiskPrintf("%s\n", dev->dev_errmsg);
+		close_device(dev, NULL);
 		return;
 	}
 	if (FAILED(IMMDeviceEnumerator_GetDevice(pEnumerator, pwszID, &DD->pDevice))) {
@@ -771,9 +891,19 @@ static void open_wasapi_playback(struct sound_dev * dev, int use_callback)
 				(int)def_period / REFTIMES_PER_MICROSEC, (int)min_period / REFTIMES_PER_MICROSEC);
 	}
 
-	if (make_format(dev, &wave_format) == 0) {
-		close_device(dev, "Device can not support the sample rate or number of channels.");
-		return;
+	if (make_format(dev, &wave_format, AUDCLNT_SHAREMODE_EXCLUSIVE) == 0) {		// failure
+		if (make_format(dev, &wave_format, AUDCLNT_SHAREMODE_SHARED) == 0) {	// failure
+			close_device(dev, "Device can not support the sample rate or number of channels.");
+			return;
+		}
+		DD->sharemode = AUDCLNT_SHAREMODE_SHARED;
+		if (quisk_sound_state.verbose_sound)
+			QuiskPrintf("  Exclusive mode refused. Open device in Shared mode\n");
+	}
+	else {
+		DD->sharemode = AUDCLNT_SHAREMODE_EXCLUSIVE;
+		if (quisk_sound_state.verbose_sound)
+			QuiskPrintf("  Open device in Exclusive mode\n");
 	}
 	if (quisk_sound_state.verbose_sound) {
 		QuiskPrintf("  Sample rate %d\n  Channel_I %d\n  Channel_Q %d\n",
@@ -783,10 +913,15 @@ static void open_wasapi_playback(struct sound_dev * dev, int use_callback)
 		//MeasureTimerPrecision();
 	}
 	if (use_callback) {
-		hnsRequestedDuration = REFTIMES_PER_MICROSEC * quisk_sound_state.data_poll_usec;
-		if (hnsRequestedDuration < min_period)
-			hnsRequestedDuration = min_period;
-		hr = IAudioClient_Initialize(DD->pAudioClient, AUDCLNT_SHAREMODE_EXCLUSIVE, AUDCLNT_STREAMFLAGS_EVENTCALLBACK,
+		if (DD->sharemode == AUDCLNT_SHAREMODE_EXCLUSIVE) {
+			hnsRequestedDuration = REFTIMES_PER_MICROSEC * quisk_sound_state.data_poll_usec;
+			if (hnsRequestedDuration < min_period)
+				hnsRequestedDuration = min_period;
+		}
+		else {
+			hnsRequestedDuration = 0;
+		}
+		hr = IAudioClient_Initialize(DD->pAudioClient, DD->sharemode, AUDCLNT_STREAMFLAGS_EVENTCALLBACK,
 			hnsRequestedDuration, hnsRequestedDuration, (WAVEFORMATEX *)&wave_format, NULL);
 		if (hr == AUDCLNT_E_BUFFER_SIZE_NOT_ALIGNED) {
 			if (quisk_sound_state.verbose_sound)
@@ -802,7 +937,7 @@ static void open_wasapi_playback(struct sound_dev * dev, int use_callback)
 				close_device(dev, "No audio client");
 				return;
 			}
-			hr = IAudioClient_Initialize(DD->pAudioClient, AUDCLNT_SHAREMODE_EXCLUSIVE, AUDCLNT_STREAMFLAGS_EVENTCALLBACK,
+			hr = IAudioClient_Initialize(DD->pAudioClient, DD->sharemode, AUDCLNT_STREAMFLAGS_EVENTCALLBACK,
 				hnsRequestedDuration, hnsRequestedDuration, (WAVEFORMATEX *)&wave_format, NULL);
 		}
 		if (hr != S_OK) {
@@ -825,8 +960,9 @@ static void open_wasapi_playback(struct sound_dev * dev, int use_callback)
 			return;
 		}
                 i = dev->latency_frames * 2 / DD->bufferSizeFrames;     // playbuf_nFrames is a multiple of bufferSizeFrames
-                if ( i < 2)
-                        i = 2;
+		i = ((i + 1) / 2) * 2;		// multiple is an even number
+                if ( i < 4)
+                        i = 4;
 		DD->playbuf_nFrames = i * DD->bufferSizeFrames;
 		DD->playbuf_buf = calloc(DD->playbuf_nFrames * dev->sample_bytes * dev->num_channels, 1);
 		atomic_store(&DD->playbuf_nWrite, i / 2 * DD->bufferSizeFrames);   // buffer starts half full
@@ -850,7 +986,7 @@ static void open_wasapi_playback(struct sound_dev * dev, int use_callback)
 	else {
 		hnsRequestedDuration = quisk_sound_state.latency_millisecs * REFTIMES_PER_MILLISEC * 2;		// reduce this if too large
 		for ( ;hnsRequestedDuration > REFTIMES_PER_MILLISEC * 20; hnsRequestedDuration -= (REFTIMES_PER_MILLISEC * 20)) {
-			hr = IAudioClient_Initialize(DD->pAudioClient, AUDCLNT_SHAREMODE_EXCLUSIVE, 0,
+			hr = IAudioClient_Initialize(DD->pAudioClient, DD->sharemode, 0,
 				hnsRequestedDuration, 0, (WAVEFORMATEX *)&wave_format, NULL);
 			if (hr == S_OK)
 				break;
@@ -1064,23 +1200,36 @@ static void midi_in_devices(PyObject * pylist)
 	}
 }
 
+#define MIDI_MAX	6000
+
+static char midi_chars[MIDI_MAX];
+static int midi_length;
+static HANDLE MIDI_mutex;
+
 static void CALLBACK MidiInProc(HMIDIIN hMidiIn, UINT wMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2)
 {
-	int msg, note, velocity;
+	int status, note, velocity;
 
 	if (wMsg == MIM_DATA) {
-		msg = (dwParam1 >> 4) & 0xF;
+		status = dwParam1 & 0xFF;
 		note = (dwParam1 >> 8) & 0x7F;
 		velocity = (dwParam1 >> 16) & 0x7F;
-		//QuiskPrintf ("instance %ld msg %d note %d veloc %d\n", dwInstance, msg, note, velocity);
+		while (WaitForSingleObject(MIDI_mutex, 0) == WAIT_TIMEOUT) ;
+		if (midi_length <= MIDI_MAX - 3) {
+			midi_chars[midi_length++] = status;
+			midi_chars[midi_length++] = note;
+			midi_chars[midi_length++] = velocity;
+		}
+		ReleaseMutex(MIDI_mutex);
 		if (note == dwInstance) {       // dwInstance is the CW note
-			if (msg == 9) {		// Note ON
+			// ignore channel number
+			if ((status & 0xF0) == 0x90) {		// Note ON
 				if (velocity)
 					quisk_midi_cwkey = 1;
 				else
 					quisk_midi_cwkey = 0;
 			}
-			else if (msg == 8) {     // Note OFF
+			else if ((status & 0xF0) == 0x80) {     // Note OFF
 				quisk_midi_cwkey = 0;
 			}
 		}
@@ -1094,7 +1243,7 @@ PyObject * quisk_wasapi_control_midi(PyObject * self, PyObject * args, PyObject
 	int client, close_port, get_event, get_in_names, get_in_devices;
 	static int midi_cwkey_note = -1;
 	char * device;
-	PyObject * pylist;
+	PyObject * pylist, * pybytes;
 	static HMIDIIN hMidiDevice = NULL;;
 
 	client = close_port = get_event = get_in_names = get_in_devices = -1;
@@ -1121,6 +1270,8 @@ PyObject * quisk_wasapi_control_midi(PyObject * self, PyObject * args, PyObject
 		return pylist;
 	}
 	if (client >= 0) {		// open client port
+		if ( ! MIDI_mutex)
+			MIDI_mutex = CreateMutex(NULL, FALSE, NULL);
 		quisk_midi_cwkey = 0;
 		if (hMidiDevice == NULL) {
 			if (midiInOpen(&hMidiDevice, client, (DWORD_PTR)(void*)MidiInProc, midi_cwkey_note, CALLBACK_FUNCTION) != MMSYSERR_NOERROR) {
@@ -1134,8 +1285,15 @@ PyObject * quisk_wasapi_control_midi(PyObject * self, PyObject * args, PyObject
 			}
 		}
 	}
-	if (get_event == 1)		// poll to get event
-		;
+	if (get_event == 1) {		// poll to get event
+		if (midi_length != 0) {
+			while (WaitForSingleObject(MIDI_mutex, 0) == WAIT_TIMEOUT) ;
+			pybytes = PyByteArray_FromStringAndSize(midi_chars, midi_length);
+			midi_length = 0;
+			ReleaseMutex(MIDI_mutex);
+			return pybytes;
+		}
+	}
 	Py_INCREF (Py_None);
 	return Py_None;
 }
